#pragma once

#include <Base_DAQ.h>
#include <Base_Configurator.h>

namespace inaf::oasbo::DAQ {

class AstriDAQ: public BaseDAQ {
protected:
    std::string getStateStr(Status);

public:
    int tel_id, origin_id, prog_id, obs_id, run_id, data_type;

    AstriDAQ();

    void start() override;
    void stop() override;
    void switchState(const Status) override;
    int deliverPacket() override;

    void updateObservers();
    void setRunID(int);
    int getRunID();
    void incrementRunID();
    std::string computeNewDestination(bool firstIdle = false);

    friend class AstriDAQBuilder;

};

class AstriDAQBuilder {
  protected:
      AstriDAQ* daq;

  public:
      AstriDAQBuilder();
      ~AstriDAQBuilder();
      void reset();
      AstriDAQBuilder* configFrom(Configurators::BaseConfigurator &conf);
      AstriDAQBuilder* setProtocol(ConnectionProtocols::BaseProtocol*);
      AstriDAQBuilder* setArchiver(Archivers::BaseArchiver*);
      AstriDAQBuilder* setProvider(Providers::BaseProvider*);
      AstriDAQBuilder* setMonitor(PacketMonitors::BasePacketMonitor*);
      AstriDAQBuilder* setPacket(PacketLib::BasePacket*);

      AstriDAQBuilder* setTelId(int);
      AstriDAQBuilder* setOriginId(int);
      AstriDAQBuilder* setProgId(int);
      AstriDAQBuilder* setObsId(int);
      AstriDAQBuilder* setRunId(int);
      AstriDAQBuilder* setDataType(int);

      AstriDAQ* getDAQ();

      static const std::string config_target;

      static const std::string protocol_key;
      static const std::string archiver_key;
      static const std::string provider_key;
      static const std::string packet_key;
      static const std::string monitor_key;

      static const std::string tcp_prot_key;
      static const std::string udp_prot_key;
      static const std::string file_arch_key;
      static const std::string kafka_prov_key;
      static const std::string redis_prov_key;
      static const std::string astri_ma_packet_key;
      static const std::string astri_horn_packet_key;
      static const std::string astri_simple_monitor_key;

      static const std::string tel_id_key;
      static const std::string origin_id_key;
      static const std::string prog_id_key;
      static const std::string obs_id_key;
      static const std::string run_id_key;
      static const std::string data_type_key;

  };

}
