//============================================================================
// Name        : Receiver.cpp
// Author      : Valerio Pastore

//============================================================================

#include <Astri_DAQ.h>
#include <Astri_Redis_DAQ_Observer.h>
#include <Yaml_Conf.h>
#include <CL_Conf.h>
#include <Mysql_Conf.h>

#include <csignal>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

using namespace inaf::oasbo;
using namespace inaf::oasbo::DAQ;
using namespace inaf::oasbo::DAQ_observers;

AstriDAQ *daq;
void my_handler(int s) {
	daq->getMonitorPtr()->printStats();
	std::cout << "Termination..." << std::endl;
	daq->switchState(AstriDAQ::STOP);

	std::cout << "Terminated by CTRL-C\n" << std::endl;
	delete daq;
	exit(1);
}


int main(int argc, char **argv) {
	Configurators::YamlConfigurator yamlconf("/home/astri/BIAS/config/DAQ/config.yaml");
	Configurators::MysqlConfigurator mysqlConf("127.0.0.1", 3306, "bias", "BIAS1ast01$1", "BIAS");
	Configurators::CLConfigurator clConf(argc,argv);

	daq = AstriDAQBuilder().configFrom(yamlconf)->configFrom(mysqlConf)->configFrom(clConf)->getDAQ();

	std::map<std::string, std::string> actConf;
	for(auto [k,v] : yamlconf.getConfig()){
		actConf[k] = v;
	}
	for(auto [k,v] : mysqlConf.getConfig()){
			actConf[k] = v;
	}
	for(auto [k,v] : clConf.getConfig()){
		actConf[k] = v;
	}

	std::cout << "Starting program with params: " << std::endl;
	for(auto [k,v] : actConf){
		std::cout << k << " : " << v << std::endl;
	}	std::cout << "-------------------------------" << std::endl;

	std::signal(SIGINT, [](int signal) {
		my_handler(signal);
	});

	daq->registerObserver(new AstriRedisDAQObserver(daq, "127.0.0.1", 6379));
	daq->start();

	std::cout << "Terminated normally\n" << std::endl;
	delete daq;

	return 0;
}
